@@ -1,38 +1,28 @@
-"""
-    sphinx.events
-    ~~~~~~~~~~~~~
+"""Sphinx core events.
 
-    Sphinx core events.
-
-    Gracefully adapted from the TextPress system by Armin.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
+Gracefully adapted from the TextPress system by Armin.
 """
 
-import warnings
 from collections import defaultdict
 from operator import attrgetter
-from typing import Any, Callable, Dict, List, NamedTuple, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, NamedTuple, Tuple, Type
 
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.errors import ExtensionError, SphinxError
 from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.inspect import safe_getattr
 
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
-
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
 
 
 logger = logging.getLogger(__name__)
 
-EventListener = NamedTuple('EventListener', [('id', int),
-                                             ('handler', Callable),
-                                             ('priority', int)])
+
+class EventListener(NamedTuple):
+    id: int
+    handler: Callable
+    priority: int
 
 
 # List of all known core events. Maps name to arguments description.
@@ -58,13 +48,10 @@ core_events = {
 class EventManager:
     """Event manager for Sphinx."""
 
-    def __init__(self, app: "Sphinx" = None) -> None:
-        if app is None:
-            warnings.warn('app argument is required for EventManager.',
-                          RemovedInSphinx40Warning)
+    def __init__(self, app: "Sphinx") -> None:
         self.app = app
         self.events = core_events.copy()
-        self.listeners = defaultdict(list)  # type: Dict[str, List[EventListener]]
+        self.listeners: Dict[str, List[EventListener]] = defaultdict(list)
         self.next_listener_id = 0
 
     def add(self, name: str) -> None:
@@ -91,7 +78,7 @@ class EventManager:
                     listeners.remove(listener)
 
     def emit(self, name: str, *args: Any,
-             allowed_exceptions: Tuple["Type[Exception]", ...] = ()) -> List:
+             allowed_exceptions: Tuple[Type[Exception], ...] = ()) -> List:
         """Emit a Sphinx event."""
         try:
             logger.debug('[app] emitting event: %r%s', name, repr(args)[:100])
@@ -104,24 +91,23 @@ class EventManager:
         listeners = sorted(self.listeners[name], key=attrgetter("priority"))
         for listener in listeners:
             try:
-                if self.app is None:
-                    # for compatibility; RemovedInSphinx40Warning
-                    results.append(listener.handler(*args))
-                else:
-                    results.append(listener.handler(self.app, *args))
+                results.append(listener.handler(self.app, *args))
             except allowed_exceptions:
                 # pass through the errors specified as *allowed_exceptions*
                 raise
             except SphinxError:
                 raise
             except Exception as exc:
+                if self.app.pdb:
+                    # Just pass through the error, so that it can be debugged.
+                    raise
                 modname = safe_getattr(listener.handler, '__module__', None)
                 raise ExtensionError(__("Handler %r for event %r threw an exception") %
                                      (listener.handler, name), exc, modname=modname) from exc
         return results
 
     def emit_firstresult(self, name: str, *args: Any,
-                         allowed_exceptions: Tuple["Type[Exception]", ...] = ()) -> Any:
+                         allowed_exceptions: Tuple[Type[Exception], ...] = ()) -> Any:
         """Emit a Sphinx event and returns first result.
 
         This returns the result of the first handler that doesn't return ``None``.
