@@ -1,17 +1,9 @@
-"""
-    sphinx.util.console
-    ~~~~~~~~~~~~~~~~~~~
-
-    Format colored console output.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Format colored console output."""
 
 import os
 import re
 import sys
-from typing import Dict
+from typing import Dict, Pattern
 
 try:
     # check if colorama is installed to support color on Windows
@@ -20,32 +12,34 @@ except ImportError:
     colorama = None
 
 
-_ansi_re = re.compile('\x1b\\[(\\d\\d;){0,2}\\d\\dm')
-codes = {}  # type: Dict[str, str]
+_ansi_re: Pattern = re.compile('\x1b\\[(\\d\\d;){0,2}\\d\\dm')
+codes: Dict[str, str] = {}
 
 
 def terminal_safe(s: str) -> str:
-    """safely encode a string for printing to the terminal."""
+    """Safely encode a string for printing to the terminal."""
     return s.encode('ascii', 'backslashreplace').decode('ascii')
 
 
 def get_terminal_width() -> int:
     """Borrowed from the py lib."""
+    if sys.platform == "win32":
+        # For static typing, as fcntl & termios never exist on Windows.
+        return int(os.environ.get('COLUMNS', 80)) - 1
     try:
-        import termios
         import fcntl
         import struct
-        call = fcntl.ioctl(0, termios.TIOCGWINSZ,  # type: ignore
-                           struct.pack('hhhh', 0, 0, 0, 0))
+        import termios
+        call = fcntl.ioctl(0, termios.TIOCGWINSZ, struct.pack('hhhh', 0, 0, 0, 0))
         height, width = struct.unpack('hhhh', call)[:2]
         terminal_width = width
     except Exception:
         # FALLBACK
-        terminal_width = int(os.environ.get('COLUMNS', "80")) - 1
+        terminal_width = int(os.environ.get('COLUMNS', 80)) - 1
     return terminal_width
 
 
-_tw = get_terminal_width()
+_tw: int = get_terminal_width()
 
 
 def term_width_line(text: str) -> str:
@@ -58,9 +52,13 @@ def term_width_line(text: str) -> str:
 
 
 def color_terminal() -> bool:
+    if 'NO_COLOR' in os.environ:
+        return False
     if sys.platform == 'win32' and colorama is not None:
         colorama.init()
         return True
+    if 'FORCE_COLOR' in os.environ:
+        return True
     if not hasattr(sys.stdout, 'isatty'):
         return False
     if not sys.stdout.isatty():
